// IO for Cells in a couple more formats.
//
// Daniel Wood.  26 May 1999.
//
// Stoled code from objWriteCell in obj.cc

#include "cell/vertex.hh"
#include "cell/face.hh"
#include "cell/cell.hh"
#include "view.hh"

void writeray( Cell *cell, View *view, ostream &s )
// Write out in the highly "proprietary" format used by the
// University of Washington 457 and 557 ray tracers circa 1999.
{
	// renumber vertices in current order
	// yuk: should really leave ids intact ???
	{
		CellVertexIterator vertices(cell);
		Vertex      *vertex;
		unsigned int id = 1;

		while( vertex = vertices.next() )
			vertex->setID(id++);
	}
	
	Vec3 eye = view->eye();
	Quaternion dir = view->dir();
	double fov = view->fov();

	s
	<< "SBT-raytracer 1.0\n"
	<< "camera {\n"
	<< "\tposition=( " << eye[0] << "," << eye[1] << "," << eye[2] << " );\n"
	<< "\tquaternion = ( " << dir[0] << "," << dir[1] << "," << dir[2] << "," << dir[3] << " );\n"
	<< "\tfov = " << fov << ";\n}\n";

	// some lights just for fun
	s
		<< "directional_light {\n"
		<< "\tdirection = (0, 0, 1);\n"
		<< "\tcolour = (0.9, 0.7, 0.6);\n"
		<< "}\n"
		<< "directional_light {\n"
		<< "\tdirection = (0, 0, -1);\n"
		<< "\tcolour = (0.9, 0.7, 0.6);\n"
		<< "}\n";

	s
	<< "polymesh {\n\tpoints = ( ";
	// write vertices in same order: explicit ids become implicit ids
	{
		CellVertexIterator vertices(cell);
		Vertex *vertex;

		bool first = true;
		while( vertex = vertices.next() )
		{
			if( !first )
				s << ",";
			first = false;
			s << "( " << vertex->pos[0] << "," << vertex->pos[1] << "," << vertex->pos[2] << " )";
		}
	}
	s << " );\n\t normals = ( ";
	{
		CellVertexIterator vertices(cell);
		Vertex *vertex;

		bool first = true;
		while( vertex = vertices.next() )
		{
			if( !first )
				s << ",";
			first = false;
			s << "( " << vertex->nor[0] << "," << vertex->nor[1] << "," << vertex->nor[2] << " )";
		}
	}

	s << " );\n\t faces = ( ";	
	// write faces in any order
	{
		CellFaceIterator faces(cell);
		Face *face;
		bool first = true;
		while( face = faces.next() )
		{
			if( !first )
				s << ",";
			first = false;
			s << "(";
			bool first2 = true;
			FaceEdgeIterator edges(face);
			Edge *edge;
			while( edge = edges.next() )
			{
				if( !first2 )
					s << ",";
				first2 = false;
				s << edge->Org()->getID()-1;
			}
			s << ")";
		}
		s << " );\n}\n";
	}
}


void writeiv( Cell *cell, View *view, ostream &s )
// Write out in the Inventor format.  No camera.
{
	// renumber vertices in current order
	// yuk: should really leave ids intact ???
	{
		CellVertexIterator vertices(cell);
		Vertex      *vertex;
		unsigned int id = 1;

		while( vertex = vertices.next() )
			vertex->setID(id++);
	}
	
#if 0
	Vec3 eye = view->eye();
	Quaternion dir = view->dir();
	double fov = view->fov();
#endif

	s << "#Inventor V2.1 ascii\n\n";

	s << "Coordinate3 {\n\tpoint [\n";


	// write vertices in same order: explicit ids become implicit ids
	{
		CellVertexIterator vertices(cell);
		Vertex *vertex;

		bool first = true;
		while( vertex = vertices.next() )
		{
			if( !first )
				s << ",";
			first = false;
			s << " " << vertex->pos[0] << " " << vertex->pos[1] << " " << vertex->pos[2] << " ";
		}
	}
	s << " ]\n}\n";

	s << "Normal {\n\tvector [\n";


	// write vertices in same order: explicit ids become implicit ids
	{
		CellVertexIterator vertices(cell);
		Vertex *vertex;

		bool first = true;
		while( vertex = vertices.next() )
		{
			if( !first )
				s << ",";
			first = false;
			s << " " << vertex->nor[0] << " " << vertex->nor[1] << " " << vertex->nor[2] << " ";
		}
	}
	s << " ]\n}\n";
	s << "IndexedFaceSet {\n\tcoordIndex [\n";
	// write faces in any order
	{
		CellFaceIterator faces(cell);
		Face *face;
		bool first = true;
		while( face = faces.next() )
		{
			FaceEdgeIterator edges(face);
			Edge *edge;
			while( edge = edges.next() )
			{
				if( !first )
					s << ",";
				first = false;
				s << edge->Org()->getID()-1;
			}
			s << ",-1";
		}
		s << " ]\n";
	}
	s << "\tnormalIndex [\n";
	// write faces in any order
	{
		CellFaceIterator faces(cell);
		Face *face;
		bool first = true;
		while( face = faces.next() )
		{
			FaceEdgeIterator edges(face);
			Edge *edge;
			while( edge = edges.next() )
			{
				if( !first )
					s << ",";
				first = false;
				s << edge->Org()->getID()-1;
			}
			s << ",-1";
		}
		s << " ]\n}\n";
	}
}